@@ -252,13 +252,14 @@ vtkInSituPipeline* vtkInSituInitializationHelper::AddPipeline(
 #else
       char splitChar = ':';
 #endif
+      std::string fileName = vtksys::SystemTools::GetFilenameName(path);
       std::vector<std::string> paths = vtksys::SystemTools::SplitString(pythonPath, splitChar);
       for (auto& p : paths)
       {
 #if defined(_WIN32) && !defined(__MINGW32__)
-        std::string testPath = p + "\\" + path;
+        std::string testPath = p + "\\" + fileName;
 #else
-        std::string testPath = p + "/" + path;
+        std::string testPath = p + "/" + fileName;
 #endif
         if (vtkPSystemTools::FileExists(testPath.c_str()))
         {
@@ -494,8 +495,21 @@ bool vtkInSituInitializationHelper::ExecutePipelines(int timestep, double time,
       item.ExecuteFailed = !item.Pipeline->Execute(timestep, time);
     }
   }
-
   internals.InExecutePipelines = false;
+
+  for (auto& item : internals.Pipelines)
+  {
+    if (item.InitializationFailed)
+    {
+      vtkLogF(ERROR, "%s pipeline failed to initialize", item.Pipeline->GetName());
+      return false;
+    }
+    if (item.ExecuteFailed)
+    {
+      vtkLogF(ERROR, "%s pipeline failed to execute", item.Pipeline->GetName());
+      return false;
+    }
+  }
   return true;
 }
 
