@@ -9,21 +9,15 @@
 #include "vtkIOSSReaderCommunication.h"
 #include "vtkIOSSUtilities.h"
 
-#include "vtkCellArrayIterator.h"
 #include "vtkCellData.h"
 #include "vtkDataArraySelection.h"
 #include "vtkDataAssembly.h"
 #include "vtkDataSet.h"
 #include "vtkExtractGrid.h"
-#include "vtkHexahedron.h"
 #include "vtkIdList.h"
 #include "vtkInformation.h"
-#include "vtkInformationIntegerKey.h"
 #include "vtkInformationVector.h"
 #include "vtkIntArray.h"
-#include "vtkLagrangeHexahedron.h"
-#include "vtkLagrangeInterpolation.h"
-#include "vtkLagrangeQuadrilateral.h"
 #include "vtkLogger.h"
 #include "vtkMultiProcessController.h"
 #include "vtkMultiProcessStream.h"
@@ -32,20 +26,26 @@
 #include "vtkPartitionedDataSet.h"
 #include "vtkPartitionedDataSetCollection.h"
 #include "vtkPointData.h"
-#include "vtkQuad.h"
 #include "vtkRemoveUnusedPoints.h"
 #include "vtkSmartPointer.h"
-#include "vtkStreamingDemandDrivenPipeline.h"
 #include "vtkStringArray.h"
 #include "vtkStructuredGrid.h"
-#include "vtkTriangle.h"
 #include "vtkUnsignedCharArray.h"
 #include "vtkUnstructuredGrid.h"
 #include "vtkVector.h"
 #include "vtkVectorOperators.h"
+
 #include "vtksys/RegularExpression.hxx"
 #include "vtksys/SystemTools.hxx"
 
+// Ioss includes
+#include <vtk_ioss.h>
+// clang-format off
+#include VTK_IOSS(Ioss_TransformFactory.h)
+// clang-format on
+
+#include <algorithm>
+
 VTK_ABI_NAMESPACE_BEGIN
 
 std::vector<int> vtkIOSSReaderInternal::GetFileIds(
@@ -119,11 +119,12 @@ bool vtkIOSSReaderInternal::UpdateDatabaseNames(vtkIOSSReader* self)
   this->Cache.Clear();
 
   // Clear old Ioss::Region's since they may not be correct anymore.
-  this->RegionMap.clear();
+  this->ReleaseRegions();
 
   auto filenames = this->FileNames;
   auto controller = self->GetController();
   const int myrank = controller ? controller->GetLocalProcessId() : 0;
+  const int ranks = controller ? controller->GetNumberOfProcesses() : 1;
 
   if (myrank == 0)
   {
@@ -137,6 +138,17 @@ bool vtkIOSSReaderInternal::UpdateDatabaseNames(vtkIOSSReader* self)
         filenames = vtkIOSSFilesScanner::GetRelatedFiles(filenames);
       }
     }
+    else if (filenames.size() == 1 && *filenames.begin() == "catalyst.bin" && ranks > 1)
+    {
+      // "catalyst.bin" is a special filename to indicate that we should read from catalyst.
+      // To make sure that each node creates a database handle to try to read something
+      // from catalyst, we need to create a "filename" for each rank.
+      filenames.clear();
+      for (int i = 0; i < ranks; ++i)
+      {
+        filenames.insert("catalyst.bin." + std::to_string(ranks) + "." + std::to_string(i));
+      }
+    }
     else if (self->GetScanForRelatedFiles())
     {
       filenames = vtkIOSSFilesScanner::GetRelatedFiles(filenames);
@@ -356,6 +368,102 @@ bool vtkIOSSReaderInternal::UpdateTimeInformation(vtkIOSSReader* self)
   return (success == 1);
 }
 
+bool vtkIOSSReaderInternal::NeedToUpdateEntityAndFieldSelections(
+  vtkIOSSReader* self, const std::vector<DatabaseHandle>& dbaseHandles)
+{
+  std::set<std::string> databaseNames;
+  for (const auto& handle : dbaseHandles)
+  {
+    databaseNames.insert(handle.first);
+  }
+
+  auto controller = self->GetController();
+  const auto rank = controller ? controller->GetLocalProcessId() : 0;
+  const auto numRanks = controller ? controller->GetNumberOfProcesses() : 1;
+
+  // This has to be done all all ranks since not all files in a database have
+  // all the blocks consequently need not have all the fields.
+  std::array<std::set<vtkIOSSUtilities::EntityNameType>, vtkIOSSReader::NUMBER_OF_ENTITY_TYPES>
+    entity_names;
+  std::array<std::set<std::string>, vtkIOSSReader::NUMBER_OF_ENTITY_TYPES> field_names;
+  std::set<vtkIOSSUtilities::EntityNameType> bc_names;
+
+  // format should have been set (and synced) across all ranks by now.
+  assert(this->Format != vtkIOSSUtilities::UNKNOWN);
+
+  for (const auto& databaseName : databaseNames)
+  {
+    auto fileids = this->GetFileIds(databaseName, rank, numRanks);
+
+    for (const auto& fileid : fileids)
+    {
+      if (auto region = this->GetRegion(databaseName, fileid))
+      {
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_node_blocks(),
+          entity_names[vtkIOSSReader::NODEBLOCK], field_names[vtkIOSSReader::NODEBLOCK]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_edge_blocks(),
+          entity_names[vtkIOSSReader::EDGEBLOCK], field_names[vtkIOSSReader::EDGEBLOCK]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_face_blocks(),
+          entity_names[vtkIOSSReader::FACEBLOCK], field_names[vtkIOSSReader::FACEBLOCK]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_element_blocks(),
+          entity_names[vtkIOSSReader::ELEMENTBLOCK], field_names[vtkIOSSReader::ELEMENTBLOCK]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_structured_blocks(),
+          entity_names[vtkIOSSReader::STRUCTUREDBLOCK],
+          field_names[vtkIOSSReader::STRUCTUREDBLOCK]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_nodesets(),
+          entity_names[vtkIOSSReader::NODESET], field_names[vtkIOSSReader::NODESET]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_edgesets(),
+          entity_names[vtkIOSSReader::EDGESET], field_names[vtkIOSSReader::EDGESET]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_facesets(),
+          entity_names[vtkIOSSReader::FACESET], field_names[vtkIOSSReader::FACESET]);
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_elementsets(),
+          entity_names[vtkIOSSReader::ELEMENTSET], field_names[vtkIOSSReader::ELEMENTSET]);
+
+        // note: for CGNS, the sidesets contain family names for BC. They need to
+        // be handled differently from exodus side sets.
+        vtkIOSSUtilities::GetEntityAndFieldNames(region, region->get_sidesets(),
+          entity_names[vtkIOSSReader::SIDESET], field_names[vtkIOSSReader::SIDESET]);
+
+        // note: for CGNS, the structuredblock elements have nested BC patches. These patches
+        // are named as well. Let's collect those names too.
+        for (const auto& sb : region->get_structured_blocks())
+        {
+          const int64_t id = sb->property_exists("id") ? sb->get_property("id").get_int() : 0;
+          for (auto& bc : sb->m_boundaryConditions)
+          {
+            if (!bc.m_bcName.empty())
+            {
+              bc_names.emplace(static_cast<vtkTypeUInt64>(id), bc.m_bcName);
+            }
+          }
+        }
+
+        // another CGNS idiosyncrasy, we need to read node fields from
+        // node_blocks nested under the structured_blocks.
+        for (auto& sb : region->get_structured_blocks())
+        {
+          std::set<vtkIOSSUtilities::EntityNameType> unused;
+          vtkIOSSUtilities::GetEntityAndFieldNames(region,
+            Ioss::NodeBlockContainer({ &sb->get_node_block() }), unused,
+            field_names[vtkIOSSReader::NODEBLOCK]);
+        }
+      }
+      // necessary to avoid errors from IO libraries, e.g. CGNS, about
+      // too many files open.
+      this->ReleaseHandles();
+    }
+  }
+
+  bool subsetOrEqual = true;
+  for (int i = 0; i < vtkIOSSReader::NUMBER_OF_ENTITY_TYPES; ++i)
+  {
+    subsetOrEqual &= std::includes(this->EntityNames[i].begin(), this->EntityNames[i].end(),
+      entity_names[i].begin(), entity_names[i].end());
+  }
+
+  return !subsetOrEqual;
+}
+
 bool vtkIOSSReaderInternal::UpdateEntityAndFieldSelections(vtkIOSSReader* self)
 {
   if (this->SelectionsMTime > this->DatabaseNamesMTime)
@@ -635,6 +743,11 @@ bool vtkIOSSReaderInternal::UpdateAssembly(vtkIOSSReader* self, int* tag)
   return true;
 }
 
+vtkDataAssembly* vtkIOSSReaderInternal::GetAssembly() const
+{
+  return this->Assembly;
+}
+
 bool vtkIOSSReaderInternal::GenerateOutput(
   vtkPartitionedDataSetCollection* output, vtkIOSSReader* self)
 {
@@ -922,6 +1035,17 @@ std::vector<vtkSmartPointer<vtkDataSet>> vtkIOSSReaderInternal::GetDataSets(
   // TODO: ideally, this method shouldn't depend on format but entity type.
   switch (this->Format)
   {
+    case vtkIOSSUtilities::DatabaseFormatType::CATALYST:
+      switch (vtk_entity_type)
+      {
+        case EntityType::STRUCTUREDBLOCK:
+        case EntityType::SIDESET:
+          return this->GetCGNSDataSets(blockname, vtk_entity_type, handle, timestep, self);
+
+        default:
+          return this->GetExodusDataSets(blockname, vtk_entity_type, handle, timestep, self);
+      }
+
     case vtkIOSSUtilities::DatabaseFormatType::CGNS:
       switch (vtk_entity_type)
       {
@@ -935,7 +1059,6 @@ std::vector<vtkSmartPointer<vtkDataSet>> vtkIOSSReaderInternal::GetDataSets(
       }
 
     case vtkIOSSUtilities::DatabaseFormatType::EXODUS:
-    case vtkIOSSUtilities::DatabaseFormatType::CATALYST:
       switch (vtk_entity_type)
       {
         case EntityType::STRUCTUREDBLOCK:
@@ -1144,7 +1267,7 @@ void vtkIOSSReaderInternal::GenerateElementAndSideIds(vtkDataSet* dataset, Ioss:
               << "[.\n";
 #endif
     // ioss element_side_raw is 1-indexed; make it 0-indexed for VTK.
-    auto transform = std::unique_ptr<Ioss::Transform>(Iotr::Factory::create("offset"));
+    auto transform = std::unique_ptr<Ioss::Transform>(Ioss::TransformFactory::create("offset"));
     transform->set_property("offset", -1);
 
     auto element_side_raw =
@@ -1946,7 +2069,7 @@ bool vtkIOSSReaderInternal::GetNodeFields(vtkDataSetAttributes* dsa,
   }
 }
 
-bool vtkIOSSReaderInternal::GenerateFileId(vtkDataSetAttributes* cd, vtkIdType numberOfCells,
+bool vtkIOSSReaderInternal::GenerateFileId(vtkDataSetAttributes* cellData, vtkIdType numberOfCells,
   Ioss::GroupingEntity* group_entity, const DatabaseHandle& handle)
 {
   if (!group_entity)
@@ -1958,7 +2081,7 @@ bool vtkIOSSReaderInternal::GenerateFileId(vtkDataSetAttributes* cd, vtkIdType n
   if (auto file_ids = vtkDataArray::SafeDownCast(cache.Find(group_entity, "__vtk_file_ids__")))
   {
     assert(numberOfCells == file_ids->GetNumberOfTuples());
-    cd->AddArray(file_ids);
+    cellData->AddArray(file_ids);
     return true;
   }
 
@@ -1984,7 +2107,7 @@ bool vtkIOSSReaderInternal::GenerateFileId(vtkDataSetAttributes* cd, vtkIdType n
 
   std::fill(file_ids->GetPointer(0), file_ids->GetPointer(0) + numberOfCells, fileId);
   cache.Insert(group_entity, "__vtk_file_ids__", file_ids.GetPointer());
-  cd->AddArray(file_ids);
+  cellData->AddArray(file_ids);
   return true;
 }
 
