@@ -410,9 +410,7 @@ PyObject* vtkMatplotlibMathTextUtilities::GetFontProperties(vtkTextProperty* tpr
 
   char tpropFamily[16];
   char tpropStyle[16];
-  char tpropVariant[16] = "normal";
   char tpropWeight[16];
-  char tpropStretch[16] = "normal";
   long tpropFontSize;
 
   switch (tprop->GetFontFamily())
@@ -450,13 +448,57 @@ PyObject* vtkMatplotlibMathTextUtilities::GetFontProperties(vtkTextProperty* tpr
   tpropFontSize = tprop->GetFontSize();
 
   vtkPythonScopeGilEnsurer gilEnsurer;
-  return PyObject_CallFunction(this->FontPropertiesClass, const_cast<char*>("sssssi"), tpropFamily,
-    tpropStyle, tpropVariant, tpropWeight, tpropStretch, tpropFontSize);
+
+  vtkSmartPyObject kwargs = PyDict_New();
+#define kwargs_insert(dict, key, value)                                                            \
+  do                                                                                               \
+  {                                                                                                \
+    if (PyDict_SetItemString(dict, key, value))                                                    \
+    {                                                                                              \
+      vtkErrorMacro(<< "Failed to set the \"" key "\" argument to `FontProperties`");              \
+      return nullptr;                                                                              \
+    }                                                                                              \
+  } while (false)
+#define kwarg_ctor_check(obj)                                                                      \
+  do                                                                                               \
+  {                                                                                                \
+    if (!obj)                                                                                      \
+    {                                                                                              \
+      vtkErrorMacro(<< "Failed to construct the \"" #obj "\" argument");                           \
+      return nullptr;                                                                              \
+    }                                                                                              \
+  } while (false)
+
+  vtkSmartPyObject kwFamily = PyUnicode_FromString(tpropFamily);
+  kwarg_ctor_check(kwFamily);
+  kwargs_insert(kwargs, "family", kwFamily);
+  vtkSmartPyObject kwStyle = PyUnicode_FromString(tpropStyle);
+  kwarg_ctor_check(kwStyle);
+  kwargs_insert(kwargs, "style", kwStyle);
+  vtkSmartPyObject kwVariant = PyUnicode_FromString("normal");
+  kwarg_ctor_check(kwVariant);
+  kwargs_insert(kwargs, "variant", kwVariant);
+  vtkSmartPyObject kwWeight = PyUnicode_FromString(tpropWeight);
+  kwarg_ctor_check(kwWeight);
+  kwargs_insert(kwargs, "weight", kwWeight);
+  vtkSmartPyObject kwStretch = PyUnicode_FromString("normal");
+  kwarg_ctor_check(kwStretch);
+  kwargs_insert(kwargs, "stretch", kwStretch);
+  vtkSmartPyObject kwFontSize = PyLong_FromLong(tpropFontSize);
+  kwarg_ctor_check(kwFontSize);
+  kwargs_insert(kwargs, "size", kwFontSize);
+
+  vtkSmartPyObject args = PyTuple_New(0);
+  kwarg_ctor_check(args);
+#undef kwarg_ctor_check
+#undef kwargs_insert
+
+  return PyObject_Call(this->FontPropertiesClass, args, kwargs);
 }
 
 //------------------------------------------------------------------------------
 void vtkMatplotlibMathTextUtilities::GetJustifiedBBox(
-  int rows, int cols, vtkTextProperty* tprop, int bbox[])
+  int rows, int cols, vtkTextProperty* tprop, int bbox[4])
 {
   bbox[0] = 0;
   bbox[1] = cols - 1;
